class TbCommerce::Transaction < ActiveRecord::Base
  self.table_name = 'tb_commerce_transactions'

  scope :search, ->(term){ where('CONCAT(billing_first_name, " ", billing_last_name) LIKE ?', "%#{term}%") }

  belongs_to :order, :inverse_of => :transactions, :foreign_key => :tb_commerce_order_id

  attr_reader :card_expiration
  attr_reader :response

  validates_presence_of :order
  validates_numericality_of :amount_charged, :greater_than => 0
  before_validation :set_amount_charged

  # Transaction status
  #
  # Status describes the current state of a transaction
  #
  # * Pending: The transaction is newly created
  # * Captured: The transaction has been submitted to the payment gateway and the purchase is finalized
  # * Fail: A failure code was received from the payment gateway
  # * Voided: The transaction is voided in the gateway
  # * Refunded: The transaction is refunded in the gateway
  #
  module Status
    PENDING = 'pending'
    CAPTURED = 'captured'
    FAIL = 'fail'
    VOIDED = 'voided'
    REFUNDED = 'refunded'
  end

  # Scopes
  scope :pending, ->{ where(:status => Status::PENDING) }
  scope :captured, ->{ where(:status => Status::CAPTURED) }
  scope :failed, ->{ where(:status => Status::FAIL) }

  def captured?
    return status == Status::CAPTURED
  end

  # Capture the transaction
  #
  #  - Create a charge in stripe
  #  - Run callbacks on the Order model
  #  - Return true if the charge was succesful
  #
  def capture!(token)
    self.with_lock(true) do

      if self.status == Status::CAPTURED
        raise StandardError, 'Transaction has already been captured'
      end

      begin
        charge = Stripe::Charge.create(build_purchase_options(token))
        self.stripe_charge_id = charge.id
        self.raw_response = charge.to_json
        self.status = Status::CAPTURED if charge.captured
        self.save()
      rescue Stripe::StripeError => e
        self.status = 'fail'
        self.raw_response = e.message
        self.save()
        self.errors.add(:base, e.message)
      end

      if self.status == Status::CAPTURED
        begin
          order.run_callbacks(:capture)
        rescue StandardError => e
          logger.fatal "Error running order callbacks: #{e.message}"
        end
        return true
      else
        return false
      end

    end
  end

  def amount_in_cents
    return (amount_charged * 100).to_i
  end

  def response_json
    begin
      return JSON.parse(raw_response)
    rescue JSON::ParseError => e
      return {}
    end
  end

private

  def build_purchase_options(token)
    return {
      :source => token,
      :amount => amount_in_cents(),
      :currency => 'usd',
      :description => self.order.description,
      :metadata => {
        :invoice_number => self.order.invoice_number
      }
    }
  end

  def set_amount_charged
    self.amount_charged = self.order.cart.total_price
    self.tax_charged = self.order.cart.sales_tax
    self.shipping_charged = self.order.cart.shipping_cost
    return true
  end

end
